-----------------------------------------------------------------------------------------------
-- Client Lua Script for SimplifiedXP
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "Apollo"
require "GameLib"
require "GroupLib"
require "PlayerPathLib"
 
-----------------------------------------------------------------------------------------------
-- SimplifiedXP Module Definition
-----------------------------------------------------------------------------------------------
local SimplifiedXP = {} 
local knMaxLevel = 50 -- TODO: Replace with a variable from code
local knMaxPathLevel = 30 -- TODO: Replace this with a non hardcoded value
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
local ktPathIcon = {
	[PlayerPathLib.PlayerPathType_Soldier] 		= "ClientSprites:Icon_Windows_UI_CRB_Soldier",
	[PlayerPathLib.PlayerPathType_Settler] 		= "ClientSprites:Icon_Windows_UI_CRB_Colonist",
	[PlayerPathLib.PlayerPathType_Scientist] 	= "ClientSprites:Icon_Windows_UI_CRB_Scientist",
	[PlayerPathLib.PlayerPathType_Explorer] 	= "ClientSprites:Icon_Windows_UI_CRB_Explorer",
}

local c_arPathStrings = {
	[PlayerPathLib.PlayerPathType_Soldier] 		= "CRB_Soldier",
	[PlayerPathLib.PlayerPathType_Settler] 		= "CRB_Settler",
	[PlayerPathLib.PlayerPathType_Scientist] 	= "CRB_Scientist",
	[PlayerPathLib.PlayerPathType_Explorer] 	= "CRB_Explorer",
}
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SimplifiedXP:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function SimplifiedXP:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- SimplifiedXP OnLoad
-----------------------------------------------------------------------------------------------
function SimplifiedXP:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("SimplifiedXP.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SimplifiedXP OnDocLoaded
-----------------------------------------------------------------------------------------------
function SimplifiedXP:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Simplified-XP-Bar-Container", nil, self)
		self.pathMain = Apollo.LoadForm(self.xmlDoc, "Simplified-Path-Bar-Container", nil, self)
		
		Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self) -- saving pos/r-size
		Apollo.RegisterEventHandler("UI_XPChanged", 				"OnXpChanged", self)
		Apollo.RegisterEventHandler("ElderPointsGained", 			"OnXpChanged", self)
	
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(true, true)
		self.pathMain:Show(true, true)
		self:OnForceXP()

		-- if the xmlDoc is no longer needed, you should set it to nil
		self.xmlDoc = nil

		self.timer = ApolloTimer.Create(3, true, "OnForceXP", self)
		
	end
end

-----------------------------------------------------------------------------------------------
-- SimplifiedXP Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on timer
function SimplifiedXP:OnForceXP()

	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		return
	end
	if GameLib.GetPlayerUnit() ~= nil then
		self:OnXpChanged()
	end
	
end


function SimplifiedXP:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "SimplifiedXP"})
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.pathMain, strName = "SimplifiedPath"})
end


function SimplifiedXP:OnXpChanged()
	
	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		return
	end
	local tStats = unitPlayer:GetBasicStats()
	if tStats ~= nil then
		self:UpdateTooltips()
		if tStats.nLevel == knMaxLevel then
			self:RedrawEP()
		else
			self:RedrawXP()
		end
		self:RedrawPathXP()
	end
	
end


function SimplifiedXP:UpdateTooltips()

	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		return
	end
	--Tooltip
	local tStats = unitPlayer:GetBasicStats()
	if tStats ~= nil then
		if tStats.nLevel == knMaxLevel then
			self.wndMain:SetTooltip("EP Tooltip Soon.TM")
		else
			local unitPlayer = GameLib.GetPlayerUnit()
			xpTooltip = self:ConfigureXPTooltip(unitPlayer)
			self.wndMain:SetTooltip(xpTooltip)
		end
		local unitPlayer = GameLib.GetPlayerUnit()
		pathTooltip = self:ConfigurePathXPTooltip(unitPlayer)
		self.pathMain:SetTooltip(pathTooltip)
	end

end


function SimplifiedXP:RedrawEP()

end


function SimplifiedXP:RedrawXP()

	local nCurrentXP = GetXp() - GetXpToCurrentLevel()
	local nNeededXP = GetXpToNextLevel()
	local nRestedXP = GetRestXp()
	local nRestedXPPool = GetRestXpKillCreaturePool() 	
	
	if not nCurrentXP or not nNeededXP or not nNeededXP or not nRestedXP then
		return
	end
	
	self.wndMain:FindChild("Simplified-XP-Bar-Container:Simplified-XP-Bar"):SetMax(nNeededXP)
	self.wndMain:FindChild("Simplified-XP-Bar-Container:Simplified-XP-Bar"):SetProgress(nCurrentXP)
	
	-- Rest Bar and Goal (where it ends)
	-- wndRestXPBarFill:SetMax(nNeededXP)
	-- wndRestXPBarFill:Show(nRestedXP and nRestedXP > 0)
	-- if nRestedXP and nRestedXP > 0 then
	-- wndRestXPBarFill:SetProgress(math.min(nNeededXP, nCurrentXP + nRestedXP))
	-- end

	local bShowRestXPGoal = nRestedXP and nRestedXPPool and nRestedXP > 0 and nRestedXPPool > 0
	self.wndMain:FindChild("Simplified-XP-Bar-Container:Simplified-Rest-Bar"):SetMax(nNeededXP)
	self.wndMain:FindChild("Simplified-XP-Bar-Container:Simplified-Rest-Bar"):Show(bShowRestXPGoal)
	if bShowRestXPGoal then
		self.wndMain:FindChild("Simplified-XP-Bar-Container:Simplified-Rest-Bar"):SetProgress(math.min(nNeededXP, nCurrentXP + nRestedXPPool))
	end

	
	return math.min(99.9, nCurrentXP / nNeededXP * 100)

end


function SimplifiedXP:RedrawPathXP()

	if not PlayerPathLib then
		return 0
	end
	
	local nCurrentLevel = PlayerPathLib.GetPathLevel()
	local nNextLevel = math.min(knMaxPathLevel, nCurrentLevel + 1)

	local nLastLevelXP = PlayerPathLib.GetPathXPAtLevel(nCurrentLevel)
	local nCurrentXP =  PlayerPathLib.GetPathXP() - nLastLevelXP
	local nNeededXP = PlayerPathLib.GetPathXPAtLevel(nNextLevel) - nLastLevelXP
	
	self.pathMain:FindChild("Simplified-Path-Bar-Container:Simplified-Path-Bar"):SetMax(nNeededXP)
	self.pathMain:FindChild("Simplified-Path-Bar-Container:Simplified-Path-Bar"):SetProgress(nCurrentXP)
	
	return math.min(99.9, nCurrentXP / nNeededXP * 100)
	
end


function SimplifiedXP:ConfigureXPTooltip(unitPlayer)

	local nCurrentXP = GetXp() - GetXpToCurrentLevel() 		-- current amount of xp into the current level
	local nNeededXP = GetXpToNextLevel() 					-- total amount needed to move through current level
	local nRestedXP = GetRestXp() 							-- amount of rested xp
	local nRestedXPPool = GetRestXpKillCreaturePool() 		-- amount of rested xp remaining from creature kills
	
	if not nCurrentXP or not nNeededXP or not nNeededXP or not nRestedXP then
		return
	end

	local strTooltip = string.format("<P Font=\"CRB_InterfaceSmall_O\">%s</P>", String_GetWeaselString(Apollo.GetString("Base_XPValue"), nCurrentXP, nNeededXP, nCurrentXP / nNeededXP * 100))
	if nRestedXP > 0 then
		local strRestLineOne = String_GetWeaselString(Apollo.GetString("Base_XPRested"), nRestedXP, nRestedXP / nNeededXP * 100)
		strTooltip = string.format("%s<P Font=\"CRB_InterfaceSmall_O\" TextColor=\"ffda69ff\">%s</P>", strTooltip, strRestLineOne)

		if nCurrentXP + nRestedXPPool > nNeededXP then
			strTooltip = string.format("%s<P Font=\"CRB_InterfaceSmall_O\" TextColor=\"ffda69ff\">%s</P>", strTooltip, Apollo.GetString("Base_XPRestedEndsAfterLevelTooltip"))
		else
			local strRestLineTwo = String_GetWeaselString(Apollo.GetString("Base_XPRestedPoolTooltip"), nRestedXPPool, ((nRestedXPPool + nCurrentXP)  / nNeededXP) * 100)
			strTooltip = string.format("%s<P Font=\"CRB_InterfaceSmall_O\" TextColor=\"ffda69ff\">%s</P>", strTooltip, strRestLineTwo)
		end
	end
	
	strTooltip = string.format("<P Font=\"CRB_InterfaceSmall_O\">%s%s</P>%s", Apollo.GetString("CRB_Level_"), unitPlayer:GetLevel(), strTooltip)
	
	return strTooltip

end


function SimplifiedXP:ConfigurePathXPTooltip(unitPlayer)
	if not PlayerPathLib then
		return ""
	end
	
	local unitPlayer = GameLib.GetPlayerUnit()
	if not unitPlayer then
		return
	end
	
	local strPathType = c_arPathStrings[unitPlayer:GetPlayerPathType()] or ""
	
	local nCurrentLevel = PlayerPathLib.GetPathLevel()
	local nNextLevel = math.min(knMaxPathLevel, nCurrentLevel + 1)

	local nLastLevelXP = PlayerPathLib.GetPathXPAtLevel(nCurrentLevel)
	local nCurrentXP =  PlayerPathLib.GetPathXP() - nLastLevelXP
	local nNeededXP = PlayerPathLib.GetPathXPAtLevel(nNextLevel) - nLastLevelXP
	
	local strTooltip = nNeededXP > 0 and string.format("<P Font=\"CRB_InterfaceSmall_O\">%s</P>", String_GetWeaselString(Apollo.GetString("Base_XPValue"), nCurrentXP, nNeededXP, nCurrentXP / nNeededXP * 100)) or ""
	strTooltip = string.format("<P Font=\"CRB_InterfaceSmall_O\">%s %s%s</P>%s", Apollo.GetString(strPathType), Apollo.GetString("CRB_Level_"), nCurrentLevel, strTooltip)
	
	return strTooltip
end

-----------------------------------------------------------------------------------------------
-- SimplifiedXP Instance
-----------------------------------------------------------------------------------------------
local SimplifiedXPInst = SimplifiedXP:new()
SimplifiedXPInst:Init()
